library("smacof")
library("ggplot2")
library("vegan")
library(mclust)
library("FactoMineR")
library(factoextra)

##################################################################################################
#
# SMACOF ANALYSIS
#
##################################################################################################

# remove elements
rm(list=ls());
# IBD matrix. This matrix comes from applying the HapIbdMain algorithm
data.t <- read.table(file="D:\\MicroArray PIRENE\\IBDMatrix.txt",header=F);
data.to <- read.table(file="D:\\MicroArray PIRENE\\Location_lat_lon_population395.txt",header=T)
# consider from the IBD matrix only the 395 individuals
ids.to.consider <- data.to$DNA_code;
consider.data.t <- rep(FALSE,nrow(data.t));
position.in.data.to <- rep(-1,nrow(data.t));
for(i in 1:nrow(data.t))
{
  iis <- which(ids.to.consider==data.t$V2[i]);
  consider.data.t[i] = sum(ids.to.consider==data.t$V2[i])==1;
  if(consider.data.t[i])
  {
    position.in.data.to[i] <- iis[1];
  }
}

# remove individuals not included in location lat lon population 395. TRUE TRUE is for retaining the first two columns
data.t <- data.t[consider.data.t,c(TRUE,TRUE,consider.data.t)]

# First two columns corresponds to labels. Retrieve the matrix of IBD similarity
m <- as.matrix(data.t[,3:ncol(data.t)])

rownames(m) <- data.t[,2];
colnames(m) <- data.t[,2];

# Remove numbers from the population labels
pop.labels <- gsub('[0-9]+', '', data.t[,2]);

bishopric.labels <- data.to$Propositus_Assigned_Bishopric[position.in.data.to[position.in.data.to>0]];

geo.coor <- cbind(data.to$Lon_X[position.in.data.to[position.in.data.to>0]],data.to$Lat_Y[position.in.data.to[position.in.data.to>0]]);

# SMACOF ON ORDINAL

mm <- m;
# not sure this really matters, but just in case: The IBS should be maximum within each individual
diag(mm) <- 0;
# transform the similarity matrix in a distance matrix based on ranks
mm.diss <- sim2diss(mm, method = "rank");

smac <- mds(mm.diss, ndim = 15);

# Stress reduction

smac$stress

smac.coord <- data.frame(smac$conf, bishopric.labels);

# swith the first coordinate, Pallars and Alt Urgell are at the left and Ripolles and Garrotxa are at the right
smac.coord[,1] <- -smac.coord[,1];

p <- ggplot(smac.coord, aes(x=D1, y=D2, color=pop.labels)) + 
  geom_point(size=6, alpha=0.6, show.legend = TRUE)

plot(p+ theme_classic() + ggtitle("Ordinal SMACOF using IBD"));

pbish <- ggplot(smac.coord, aes(x=D1, y=D2, color=bishopric.labels)) + 
  geom_point(size=6, alpha=0.6, show.legend = TRUE)

plot(pbish+ theme_classic() + ggtitle("Ordinal SMACOF using IBD"));

# How many clusters exist assuming a mixture of Gaussian distributions?

mm <- Mclust(smac.coord[,1:2], G=1:20)

bishopric.mclust <- paste(bishopric.labels,mm$classification,sep="_");

smac.coord <- data.frame(individu = data.t[,2],smac.coord, mclust.classification = mm$classification);

smac.coord$mclust.classification <- as.factor(smac.coord$mclust.classification)

pbish.mclust <- ggplot(smac.coord, aes(x=D1, y=D2, color= bishopric.labels, shape = mclust.classification)) + 
  geom_point(size=6, alpha=0.6, show.legend = TRUE)

plot(pbish.mclust+ theme_classic() + ggtitle("Ordinal SMACOF using IBD"));

#Classification Mclust/comarca
table(mm$classification, bishopric.labels)

write.table(file = "D:\\MicroArray PIRENE\\SMACOF_IBDMatrix.txt", smac.coord,row.names = FALSE)


# compare the smac.coords with the spatial coordinates of the samples

geo.table <- read.table(file="C:\\Users\\olao\\OneDrive - CRG - Centre de Regulacio Genomica\\Projects\\PIRENE project MicroArrays\\Results\\Population_Substructure\\Geographic_location.txt",header=T)

smac.coord.comparable <- c();
geo.coord.comparable <- c();

for(i in 1:nrow(data.t))
{
  exist.coord <- geo.table[,1] == data.t[i,2];
  if(sum(exist.coord)>0)
  {
    smac.coord.comparable<- c(smac.coord.comparable,i);
    geo.coord.comparable <- c(geo.coord.comparable,which(exist.coord==TRUE));
  }
}


#protest(as.matrix(data.t[,6:7]),as.matrix(data.t[,10:11]), permutations = 10000)


###############################################################################
#
# PARTIAL MANTEL TEST
#
###############################################################################
library(geodist);
m.bishop <- matrix(nrow=nrow(data.t),ncol=nrow(data.t),0);
for(i in 1:(nrow(data.t)-1))
{
  for(i2 in (i+1):nrow(data.t))
  {
    if(bishopric.labels[i]!=bishopric.labels[i2])
    {
      m.bishop[i,i2] <- 1;
      m.bishop[i2,i] <- 1;
    }
  }  
}

m.comarca <- matrix(nrow=nrow(data.t),ncol=nrow(data.t),0);
for(i in 1:(nrow(data.t)-1))
{
  for(i2 in (i+1):nrow(data.t))
  {
    if(pop.labels[i]!=pop.labels[i2])
    {
      m.comarca[i,i2] <- 1;
      m.comarca[i2,i] <- 1;
    }
  }  
}

m.comarca_bishop <- m.bishop + m.comarca;

m.comarca_bishop[m.comarca_bishop > 1] <- 1

m.geo <- as.matrix(geodist(geo.coor, measure = "haversine")/1000);
m.gen <- as.matrix(dist(smac$conf[,1:2]));

mantel.partial(m.gen, m.bishop, m.geo, method="pearson", permutations=999);
mantel.partial(m.gen, m.comarca, m.geo, method="pearson", permutations=999);
mantel.partial(m.gen, m.comarca_bishop, m.geo, method="pearson", permutations=999)


##################################################################################
#
# GEOGRAPHIC DISTANCE VS GENETIC DISTANCE BY BISHOPRICS
#
##################################################################################

library("gdata")
library(geodist)

bishoprics <- unique(bishopric.labels);
colnames(geo.coor) <- c("lon","lat")

for(i in 1:length(bishoprics))
{
  sol <- which(bishopric.labels==bishoprics[i])
  m.geo <- as.matrix(geodist(geo.coor[sol,], measure = "haversine")/1000);
  m.gen <- as.matrix(dist(smac$conf[sol,1:2]));
  print(bishoprics[i]);
  select.geo <- upperTriangle(m.geo) < 30;
  #print(mantel(m.geo[sol,sol],m.gen[sol,sol]));
  
  plot(upperTriangle(m.geo)[select.geo],upperTriangle(m.gen)[select.geo], main = bishoprics[i]);
  print(summary(lm(upperTriangle(m.gen)[select.geo] ~ upperTriangle(m.geo)[select.geo])));
}


###############################################################
#
# gegraphic distance between couples
#
###############################################################

data.to$Longitude_F <- as.numeric(data.to$Longitude_F);
data.to$Longitude_M <- as.numeric(data.to$Longitude_M);

data.to$Longitude_GF_M <- as.numeric(data.to$Longitude_GF_M);
data.to$Longitude_GM_M <- as.numeric(data.to$Longitude_GM_M);

data.to$Longitude_GF_F <- as.numeric(data.to$Longitude_GF_M);
data.to$Longitude_GM_F <- as.numeric(data.to$Longitude_GM_M);

data.to$Latiude_F <- as.numeric(data.to$Latitude_F);
data.to$Latitude_M <- as.numeric(data.to$Latitude_M);

data.to$Latitude_GF_M <- as.numeric(data.to$Latitude_GF_M);
data.to$Latitude_GM_M <- as.numeric(data.to$Latitude_GM_M);

data.to$Latitude_GF_F <- as.numeric(data.to$Latitude_GF_M);
data.to$Latitude_GM_F <- as.numeric(data.to$Latitude_GM_M);

pick <- data.to$Propositus_Assigned_Bishopric=="Vic"

grand.parents.distance <- geodist(cbind(data.to$Longitude_GF_F[pick], data.to$Latitude_GF_F[pick]), cbind(data.to$Longitude_GM_F[pick], data.to$Latitude_GM_F[pick]), paired = T, measure = "haversine")/1000; 

grand.parents.distance <- c(grand.parents.distance, geodist(cbind(data.to$Longitude_GF_M[pick], data.to$Latitude_GF_M[pick]), cbind(data.to$Longitude_GM_M[pick], data.to$Latitude_GM_M[pick]), paired = T, measure = "haversine")/1000); 

grand.parents.distance[grand.parents.distance> 200] <- NA

parents.distance <- geodist(cbind(data.to$Longitude_F[pick],data.to$Latitude_F[pick]), cbind(data.to$Longitude_M[pick],data.to$Latitude_M[pick]), paired = T, measure = "haversine")/1000;

parents.distance[parents.distance>200] <- NA

boxplot((grand.parents.distance),(parents.distance));

###############################################################
# 
#
# Procrust analysis using SMACOF result
###############################################################

procrust <- protest(smac.coord[smac.coord.comparable,1:2], geo.table[geo.coord.comparable,2:3])

procrust

# Geographic clusters out of the smac.coord.comparable

smac.coord.comp <- smac.coord[smac.coord.comparable,1:2];
geo.coord.comp <- geo.table[geo.coord.comparable,2:3];

mm.comp <- Mclust(smac.coord.comp[,1:2], G=1:10)

# plot genetic clusters in geography

my.coord <- data.frame(geo.coord.comp);

# categories

my.cat<- paste(mm.comp$classification,pop.labels[smac.coord.comparable]);

pg <- ggplot(my.coord, aes(x=Long, y=Lat, color=my.cat)) + 
  geom_point(size=6, alpha=0.6, show.legend = TRUE)

plot(pg+ theme_classic() + ggtitle("MClust in geography"));


#############################################################################
# Procrustes with PCA results
#
# partial mantel test
#
#############################################################################

rm(list=ls());
data.t <- read.table(file="C:\\Users\\olao\\Downloads\\GENPIR_PCA_BISHO_COMARCA.txt",header=T)
not.nas <- rowSums(is.na(data.t))==0;
data.t <- data.t[not.nas,]
names(data.t)

#protest(as.matrix(data.t[,6:7]),as.matrix(data.t[,10:11]), permutations = 10000)
m.bishop <- matrix(nrow=nrow(data.t),ncol=nrow(data.t),0);
for(i in 1:(nrow(data.t)-1))
{
  for(i2 in (i+1):nrow(data.t))
  {
    if(data.t$Bishopric[i]!=data.t$Bishopric[i2])
    {
      m.bishop[i,i2] <- 1;
      m.bishop[i2,i] <- 1;
    }
  }  
}

m.comarca <- matrix(nrow=nrow(data.t),ncol=nrow(data.t),0);
for(i in 1:(nrow(data.t)-1))
{
  for(i2 in (i+1):nrow(data.t))
  {
    if(data.t$COMARCA[i]!=data.t$COMARCA[i2])
    {
      m.comarca[i,i2] <- 1;
      m.comarca[i2,i] <- 1;
    }
  }  
}

m.comarca_bishop <- m.bishop + m.comarca;

m.comarca_bishop[m.comarca_bishop > 1] <- 1

m.geo <- as.matrix(dist(data.t[,5:6]));
m.gen <- as.matrix(dist(data.t[,7:8]));

mantel.partial(m.gen, m.bishop, m.geo, method="pearson", permutations=999);
mantel.partial(m.gen, m.comarca, m.geo, method="pearson", permutations=999);
mantel.partial(m.gen, m.comarca_bishop, m.geo, method="pearson", permutations=999)


bishop.same.pair <- c()
genetic.distance.same.pair <- c()
geographic.distance.same.pair <- c()

for(i in 1:(nrow(data.t)-1))
{
  for(i2 in (i+1):nrow(data.t))
  {
    if(data.t$bisbat[i]==data.t$bisbat[i2])
    {
      bishop.same.pair <- c(bishop.same.pair, data.t$bisbat[i])
      genetic.distance.same.pair <- c(genetic.distance.same.pair, m.gen[i,i2]);
      geographic.distance.same.pair <- c(geographic.distance.same.pair, m.geo[i,i2]);
    }
  }  
}


##################################################################
#
# Multivariate Correspondence Analysis
#
##################################################################
library("FactoMineR")
library(factoextra)

rm(list=ls())

data.t <- read.table(file="C:\\Users\\olao\\OneDrive - CRG - Centre de Regulacio Genomica\\Projects\\PIRENE project MicroArrays\\Results\\MCA_on_bishoprics\\Bishoprics_of_samples.txt",header=T);

data.to <- read.table(file="C:\\Users\\olao\\OneDrive - CRG - Centre de Regulacio Genomica\\Projects\\PIRENE project MicroArrays\\Results\\final_IDS_to_be_USED.txt",header=T);

ids.to.consider <- data.to$IID;

consider.data.t <- rep(FALSE,nrow(data.t));
for(i in 1:nrow(data.t))
{
  consider.data.t[i] = sum(ids.to.consider==data.t$DNA_code[i])==1
}

data.t <- data.t[consider.data.t,];

data.t <- data.t[rowSums(is.na(data.t[,2:ncol(data.t)]))==0,]

res.mca <- MCA(data.t[,2:ncol(data.t)], graph = FALSE)

fviz_screeplot(res.mca, addlabels = TRUE, ylim = c(0, 45))

options(ggrepel.max.overlaps = Inf)
fviz_mca_var(res.mca, axes = c(1,2), col.var = "cos2",
             gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"), 
             repel = TRUE, # Avoid text overlapping
             ggtheme = theme_minimal())

fviz_mca_var(res.mca, 
             repel = TRUE, # Avoid text overlapping (slow)
             ggtheme = theme_minimal())



##############################################################
#
# CA haplotypes
#
##############################################################

rm(list=ls());
data.t <- read.table(file="C:\\Users\\olao\\Downloads\\Y_haplog_freq.txt",header=T)

#data.t[data.t$Haplogroup=="H",] <- 0
#data.t[data.t$Haplogroup=="L",] <- 0
#data.t[data.t$Haplogroup=="Q",] <- 0


md <- t(as.matrix(data.t[,2:ncol(data.t)]));

colnames(md) <- data.t[,1]


data.t <- read.table(file="C:\\Users\\olao\\Downloads\\MIT_freq.txt",header=T)

md <- as.matrix(data.t[,4:ncol(data.t)]);
rownames(md) <- data.t[,1]


run.ca <- function(md, title.plot)
{
  res.ca <- CA(md, graph = FALSE)
  
  fviz_ca_biplot(res.ca, 
                 arrow = c(FALSE, TRUE),
                 repel = TRUE, title = paste("Symmetric CA Biplot", title.plot, sep = " ")) + theme(text = element_text(size = 15),
                                                                      axis.title = element_text(size = 10),
                                                                      axis.text = element_text(size = 10))
  
}

run.ca(md, "mtDNA")
